#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# reflink documentation build configuration file, created by
# sphinx-quickstart on Tue Jul  9 22:26:36 2013.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys
import os

# Get the project root dir, which is the parent dir of this
cwd = os.getcwd()
project_root = os.path.dirname(cwd)

# Insert the project root dir as the first element in the PYTHONPATH.
# This lets us ensure that the source package is imported, and that its
# version is used.
sys.path.insert(0, project_root)

# -- General configuration ---------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.viewcode']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'Python reflink'
copyright = u"2017, Ruben De Smet"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'


# -- Options for HTML output -------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'default'

# Add any paths that contain custom static files (such as style sheets)
# here, relative to this directory. They are copied after the builtin
# static files, so a file named "default.css" will overwrite the builtin
# "default.css".
html_static_path = ['_static']

# Output file base name for HTML help builder.
htmlhelp_basename = 'reflinkdoc'


# -- Options for LaTeX output ------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #'preamble': '',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    ('index', 'reflink.tex',
     u'Python reflink Documentation',
     u'Ruben De Smet', 'manual'),
]

# The name of an image file (relative to this directory) to place at
# the top of the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings
# are parts, not chapters.
#latex_use_parts = False

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True


# -- Options for manual page output ------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'reflink',
     u'Python reflink Documentation',
     [u'Ruben De Smet'], 1)
]

# If true, show URL addresses after external links.
#man_show_urls = False


# -- Options for Texinfo output ----------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    ('index', 'reflink',
     u'Python reflink Documentation',
     u'Ruben De Smet',
     'reflink',
     'One line description of project.',
     'Miscellaneous'),
]

# Documents to append as an appendix to all manuals.
#texinfo_appendices = []

# If false, no module index is generated.
#texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
#texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
#texinfo_no_detailmenu = False


autodoc_mock_imports = ['reflink.backend', 'reflink._backend', 'reflink.linux']



# -*- coding: utf-8 -*-

"""Top-level package for Python reflink."""

__author__ = """Ruben De Smet"""
__email__ = 'pypi@rubdos.be'
__version__ = '0.1.3'

from .reflink import reflink, supported_at
from .error import ReflinkImpossibleError



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `reflink` package."""

import pytest
import os
import subprocess
import random
import string
import unittest
import sys


from reflink import reflink, ReflinkImpossibleError, supported_at


@pytest.fixture
def btrfs_volume(tmpdir):
    path = tmpdir.join("volume")
    with open(path, 'bw+') as f:
        f.truncate(512 * 1024 ** 2)

    process = subprocess.call(["mkfs.btrfs", path])
    assert process == 0

    yield path
    os.remove(path)

@pytest.fixture
def btrfs_mount(tmpdir, btrfs_volume):
    mount = tmpdir.join("mount")
    os.mkdir(mount)
    process = subprocess.call(["mount", btrfs_volume, mount])
    assert process == 0

    yield mount

    process = subprocess.call(["umount", btrfs_volume])
    assert process == 0
    os.rmdir(mount)

@pytest.fixture
def file_on_mount(btrfs_mount):
    name = ''.join(random.choices(string.ascii_lowercase, k=8))
    path = os.path.join(btrfs_mount, name)
    with open(path, 'bw+') as f:
        f.write(''.join(random.choices(string.ascii_lowercase, k=100)).encode('utf-8'))

    yield path

    os.remove(path)


@pytest.mark.btrfs
def test_reflink(btrfs_mount, file_on_mount):
    target = os.path.join(btrfs_mount, "target")
    reflink(file_on_mount, target)

    assert os.path.isfile(target)

@pytest.mark.btrfs
def test_reflink(btrfs_mount):
    source = os.path.join(btrfs_mount, "source")
    target = os.path.join(btrfs_mount, "target")
    with pytest.raises(FileNotFoundError):
        reflink(source, target)

    assert not os.path.isfile(source)
    assert not os.path.isfile(target)

@pytest.mark.btrfs
def test_reflink_across_devices(btrfs_mount, file_on_mount, tmpdir):
    target = os.path.join(tmpdir, "target")

    with pytest.raises(ReflinkImpossibleError):
        reflink(file_on_mount, target)

    assert not os.path.isfile(target)

@pytest.mark.btrfs
def test_supported_at_btrfs(btrfs_mount):
    assert supported_at(btrfs_mount)

@pytest.mark.btrfs
def test_btrfs_permissions(btrfs_mount, file_on_mount):
    target = os.path.join(btrfs_mount, "target")
    reflink(file_on_mount, target)

    a = os.stat(file_on_mount)
    b = os.stat(target)
    assert a.st_mode == b.st_mode

@unittest.skipUnless(sys.platform in ['linux', 'freebsd'], "No /dev support")
def test_not_supported_at_dev():
    assert not supported_at("/dev")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

import sys


from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'cffi',
]

setup_requirements = [
    'cffi',
    'pytest-runner',
]

test_requirements = [
    'pytest',
]

if sys.platform not in ['linux', 'linux2', 'win32', 'darwin']:
    raise NotImplementedError("Platform %s not supported" % sys.platform)

if __name__ == '__main__':
    setup(
        name='reflink',
        version='0.2.0',
        description="Python reflink wraps around platform specific reflink implementations",
        long_description=readme + '\n\n' + history,
        author="Ruben De Smet",
        author_email='pypi@rubdos.be',
        url='https://gitlab.com/rubdos/pyreflink',
        packages=find_packages(include=['reflink']),
        include_package_data=True,
        install_requires=requirements,
        license="MIT license",
        zip_safe=False,
        keywords='reflink',
        classifiers=[
            'Development Status :: 2 - Pre-Alpha',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Natural Language :: English',
            "Programming Language :: Python :: 2",
            'Programming Language :: Python :: 2.6',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
        ],
        test_suite='tests',
        tests_require=test_requirements,
        setup_requires=setup_requirements,
        cffi_modules = ["reflink/native.py:ffibuilder"]
    )
